package wzmasterserver;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

public class GameManager {
    private static GameManager self;
    
    Map<String, Game> openGames, startedGames;
    
    Cleaner openGamesCleaner, startedGamesCleaner;
    
    private GameManager(){
        openGames = new HashMap();
        startedGames = new HashMap();
        
        openGamesCleaner = new Cleaner(openGames);
        startedGamesCleaner = new Cleaner(startedGames);
        openGamesCleaner.start();
        startedGamesCleaner.start();
    }
    
    public static GameManager getInstance(){
        if(self == null)self = new GameManager();
        return self;
    }
    
    public void openGame(Game game){
        openGames.put(game.getHost()+game.getPort(), game);
    }
    
    public void startGame(String host, int port){
        String key = host+port;
        Game game = openGames.get(key);
        game.setStatus(Status.STARTED);
        startedGames.put(key, game);
        openGames.remove(key);
    }
    
    public void closeGame(String host, int port){
        String key = host+port;
        startedGames.remove(key);
    }
    
    public boolean hasOpenGame(String host, int port){
        String key = host+port;
        return openGames.containsKey(key);
    }
    
    public boolean hasStartedGame(String host, int port){
        String key = host+port;
        return startedGames.containsKey(key);
    }
    
    public Game[] getAllGames(){
        ArrayList<Game> allGames = new ArrayList(openGames.size()+startedGames.size());
        allGames.addAll(openGames.values());
        allGames.addAll(startedGames.values());
        
        return allGames.toArray(new Game[0]);
    }
    
    public Game[] getGameByMatch(String[] matchers){
        Game[] games = new Game[0];
        
        if(matchers[0].equals("*"))games = getAllGames();
        
        return games;
    }
    
    class Cleaner extends Thread{
        Map<String, Game> subject;
        
        Cleaner(Map<String, Game> subject){
            super("Cleaner");
            this.subject = subject;
            this.setDaemon(true);
		}
        
        public void run(){
            System.out.println("starting cleaner thread.");
            try {
                long hour = 1000*60*60;
                while(!this.isInterrupted()){
                    this.sleep(6*hour);
                    
                    clean();
                }
            }catch(InterruptedException e){
                ; // do nothing.
            }finally{
                System.out.println("stopping cleaner thread.");
            }
        }
        
        void clean(){
            Iterator<Game> itr = subject.values().iterator();
            long now = System.currentTimeMillis();
            long _24_hours_ago = now - (1000*60*60*24);
            
            while(itr.hasNext()){
                if(itr.next().getTimestamp() < _24_hours_ago){
                    itr.remove();
                }
            }
        }
    }
}
