package wzmasterserver;

import java.io.BufferedReader;

import java.io.BufferedWriter;

import java.io.IOException;
import java.io.InputStreamReader;

import java.io.OutputStreamWriter;

import java.io.PrintWriter;

import java.net.InetAddress;
import java.net.Socket;

import java.nio.charset.Charset;

public class Session extends Thread{
    Socket client;
    Session(Socket client) {
        this.client = client;
        System.out.println("Incomming connection from "+client.getInetAddress().getHostAddress());
    }
    
    /**
     * protocol implementation!!
     */
    public void run() {
        // oh no, not another abomination of exceptions...
        BufferedReader reader = null;
        BufferedWriter writer = null;
        try {
            reader = new BufferedReader(new InputStreamReader(client.getInputStream(), Charset.forName("UTF-8")));
            writer = new BufferedWriter(new OutputStreamWriter(client.getOutputStream(), Charset.forName("UTF-8")));
            
            while(client.isConnected()){
                String readout = reader.readLine();
                try{
                    processCommand(readout, writer);
                }catch(Exception e){
                    System.err.println("ERROR IN COMMAND PROCESSING");
                    e.printStackTrace();
                    writer.append("!\tINTERNAL SERVER ERROR:");
                    e.printStackTrace(new PrintWriter(writer));
                }
            }
            
        } catch (IOException e) {
            System.err.println("Failed to open streams to client. Reason: ");
            e.printStackTrace();
            try {
                client.close();
            } catch (IOException f) {
                ;// do nothing
            }
            return;
        }
    }
    
    public void processCommand(String readout, BufferedWriter writer) throws IOException {
        if(readout != null){
            String[] command = readout.split("\t");
            String cmdName = command[0].toLowerCase();
            String reply = ErrorConstants.COMMAND_NOT_UNDERSTOOD;
            
            if(cmdName.equals("list")){
                reply = listReply(command);
            }else if(cmdName.equals("opengame")){
                reply = opengameReply(command);
            }else if(cmdName.equals("startgame")){
                reply = startgameReply(command);
            }else if(cmdName.equals("closegame")){
                reply = closegameReply(command);
            }
            
            writer.append(reply);
            writer.flush(); // don't ever forget this!!
        }
    }
    
    public String listReply(String[] command){
        if(command.length != 2)return ErrorConstants.BAD_LIST_PARAMETER_COUNT;
        
        String[] params = new String[command.length - 1];
        System.arraycopy(command,1,params,0,params.length);
        Game[] games = GameManager.getInstance().getGameByMatch(params);
        
        StringBuilder sb = new StringBuilder();
        sb.append("COUNT\t");
        sb.append(games.length);
        sb.append("\n");
        
        for(Game game : games){
            sb.append(game.toString());
            sb.append("\n");
        }
        return sb.toString();
    }

    private String opengameReply(String[] command) {
        String reply = ErrorConstants.UNKNOWN;
        if(command.length == 5){
            Game game = new Game();
            // ip, port, game-name, map-name, max-players, status
            game.setHost(client.getInetAddress().getHostAddress());
            try{
                game.setPort(Integer.parseInt(command[1]));
            }catch(NumberFormatException e){
                return ErrorConstants.BAD_PORT_NUMBER_FORMAT;
            }
            game.setGameName(command[2]);
            game.setMapName(command[3]);
            try {
                game.setMaxPlayers(Integer.parseInt(command[4]));
            }
            catch (NumberFormatException e) {
                return ErrorConstants.BAD_MAX_PLAYER_NUMBER_FORMAT;
            }
            game.setStatus(Status.OPEN);
            
            
            GameManager.getInstance().openGame(game);
            reply = "DONE\n";
        }else{
            reply = ErrorConstants.BAD_OPENGAME_PARAMETER_COUNT;
        }
        return reply;
    }

    private String startgameReply(String[] command) {
        String reply = ErrorConstants.UNKNOWN;
        if (command.length == 2) {
            GameManager manager = GameManager.getInstance();
            String host = client.getInetAddress().getHostAddress();
            int port = 0;
            
            try{
                port = Integer.parseInt(command[1]);
            }catch(NumberFormatException e){
                reply = ErrorConstants.BAD_PORT_NUMBER_FORMAT;
            }
            if(manager.hasOpenGame(host, port)){
                manager.startGame(host, port);
                reply = "DONE\n";
            }else{
                reply = ErrorConstants.NO_SUCH_GAME_REGISTERED_AS_OPEN;
            }
        }
        else {
            reply = ErrorConstants.BAD_STARTGAME_PARAMETER_COUNT;
        }
        
        return reply;
    }

    private String closegameReply(String[] command) {
        String reply = ErrorConstants.UNKNOWN;
        if (command.length == 2) {
            GameManager manager = GameManager.getInstance();
            String host = client.getInetAddress().getHostAddress();
            int port = 0;
            
            try{
                port = Integer.parseInt(command[1]);
            }catch(NumberFormatException e){
                reply = ErrorConstants.BAD_PORT_NUMBER_FORMAT;
            }
            if(manager.hasStartedGame(host, port)){
                manager.closeGame(host, port);
                reply = "DONE\n";
            }else{
                reply = ErrorConstants.NO_SUCH_GAME_REGISTERED_AS_OPEN;
            }
        }
        else {
            reply = ErrorConstants.BAD_CLOSEGAME_PARAMETER_COUNT;
        }
        
        return reply;
    }
}
